=head1 NAME

Foo::Shizzle - Get the Foo out of the Shizzle!

=head1 SYNOPSIS

  my $shiz = Foo::Shizzle->new;

=head1 DESCRIPTION

This is the description. Blah blah blah. Fo shizzle.

=head2 Interface

This is all about the interface.

=head3 C<new>

Gotta cover the C<new()> method.
=head1 AUTHOR

Someone Else

=head1 COPYRIGHT

Copyright (c) Someone Else 2009, some rights reserved.

This module is free software; you can redistribute it and/or modify it under
the same terms as Perl itself.

=cut

