our $VERSION = '0.01';

=head1 NAME

Foo::Bar - Get the Foo out of the Bar!

=head1 SYNOPSIS

  my $bar = Foo::Bar->new;
  while (my $drink = $bar->gulp) {
      say 'Belch!';
  }

=head1 DESCRIPTION

This is the description. Blah blah blah.

=head2 Interface

This is all about the interface.

=head3 C<new>

Gotta cover the C<new()> method.

=head3 C<gulp>

Thirsty?

=head1 AUTHOR

Someone Else

=head1 COPYRIGHT

Copyright (c) Someone Else 2009, some rights reserved.

This module is free software; you can redistribute it and/or modify it under
the same terms as Perl itself.

=cut

package Foo::Bar;

our $VERSION = '0.41';
